package ConverterLib;

import java.util.Comparator;

public abstract class Converter implements Comparable<Converter>, MetricConverter {

	protected ConverterType type = null;
	protected String fromUnit = null;
	protected String toUnit = null;
	protected double fromValue = 0.0;
	protected double toValue = 0.0;

	public ConverterType getConverterType() {
		return type;
	}
	
	public String getFromUnit() {
		return fromUnit;
	}
	
	public String getToUnit() {
		return toUnit;
	}
	
	public double getFromValue() {
		return fromValue;
	}
	
	public double getToValue() {
		return toValue;
	}

	public abstract void convert();
	public abstract double convert(String fromUnit, String toUnit, double fromValue);

	@Override
	public int compareTo(Converter other) {
		return this.type.getType() - other.type.getType();
	}
	public static Comparator<Converter> FromValueComparator = new Comparator<Converter>() {
		public int compare(Converter c1, Converter c2) {    
			return Double.compare(c1.getFromValue(), c2.getFromValue()); //ascending order
		}
	};

	public static Comparator<Converter> ToValueComparator = new Comparator<Converter>() {
		public int compare(Converter c1, Converter c2) {     
			return Double.compare(c1.getToValue(), c2.getToValue()); //ascending order
		}
	};
    
	@Override
    public String toString() {
   		return String.format("%s,%s=%f,%s=%f", type, fromUnit, fromValue, toUnit, toValue);
    }
	
	@Override // hashCode method override of Object class
    public int hashCode() { 
        return this.type.hashCode() * this.fromUnit.hashCode() * Double.valueOf(this.fromValue).hashCode() * this.toUnit.hashCode() * Double.valueOf(this.toValue).hashCode();
    }
    
    @Override // equals method override of Object class
    public boolean equals(Object obj) { 
    	if (obj instanceof Converter)
            return equals((Converter)obj);
        return false;
    }
	// equals method
    public boolean equals(Converter other) { 
    	if (this.type == other.type && this.fromUnit.equals(other.fromUnit) && this.fromValue == other.fromValue && this.toUnit.equals(other.toUnit) && this.toValue == other.toValue)
            return true;
        return false;
    }

}
